package com.atguigu.spzx.manger.service.impl;

import com.atguigu.constant.ExceptionConstant;
import com.atguigu.constant.MessageConstant;
import com.atguigu.exception.BunnyException;
import com.atguigu.lib.MD5;
import com.atguigu.spzx.manger.mapper.SysUserMapper;
import com.atguigu.spzx.manger.service.SysUserService;
import com.atguigu.spzx.model.dto.system.LoginDto;
import com.atguigu.spzx.model.dto.system.SysUserDto;
import com.atguigu.spzx.model.entity.system.SysUser;
import com.atguigu.spzx.model.vo.result.ResultCodeEnum;
import com.atguigu.spzx.model.vo.system.LoginVo;
import com.atguigu.utils.StringEmptyUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class SysUserServiceImpl implements SysUserService {
    @Autowired
    StringEmptyUtil stringEmptyUtil;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * * 登录请求实现
     *
     * @param loginDto 登录请求参数
     * @return 登录后结果
     */
    @Override
    public LoginVo login(LoginDto loginDto) {
        String password = loginDto.getPassword();
        String userName = loginDto.getUserName();
        String captcha = loginDto.getCaptcha();
        String key = loginDto.getCodeKey();
        // 得到Redis中验证码
        String code = (String) redisTemplate.opsForValue().get(key);
        stringEmptyUtil.isEmpty(userName, ExceptionConstant.USERNAME_IS_EMPTY);
        stringEmptyUtil.isEmpty(password, ExceptionConstant.PASSWORD_IS_EMPTY);
        stringEmptyUtil.isEmpty(captcha, ExceptionConstant.CAPTCHA_IS_EMPTY);
        stringEmptyUtil.isEmpty(key, ExceptionConstant.KEY_IS_EMPTY);
        stringEmptyUtil.isEmpty(code, ExceptionConstant.VERIFICATION_CODE_IS_EMPTY);

        // 验证码不匹配
        assert code != null;
        if ((!Objects.equals(code.toLowerCase(), captcha.toLowerCase()))) {
            throw new BunnyException(ExceptionConstant.VERIFICATION_CODE_DOES_NOT_MATCH);
        }
        // 比较完成后删除验证码
        redisTemplate.delete(key);
        // 根据username查询用户信息
        SysUser sysUser = sysUserMapper.selectByUsername(userName);
        if (sysUser == null) {
            throw new BunnyException(ExceptionConstant.USER_NOT_FOUND);
        }
        String encryptedPassword = MD5.encrypt(password);
        // 比较密码
        if (!encryptedPassword.equals(sysUser.getPassword())) {
            throw new BunnyException(ExceptionConstant.PASSWORD_ERROR);
        }
        // 登录成功
        String token = UUID.randomUUID().toString().replaceAll("-", "");
        redisTemplate.opsForValue().set(token, sysUser, 7, TimeUnit.DAYS);
        // 返回loginVo对象
        return LoginVo.builder().token(token).build();
    }

    /**
     * * 用户退出接口
     *
     * @param token token值
     */
    @Override
    public void logout(String token) {
        redisTemplate.delete(token);
    }

    /**
     * 查询用户
     *
     * @param sysUserDto 用户请求体
     * @param pageNum    当前页
     * @param pageSize   一页大小
     * @return 用户分页结果
     */
    @Override
    public PageInfo<SysUser> findByPage(SysUserDto sysUserDto, Integer pageNum, Integer pageSize) {
        Page<Object> startedPage = PageHelper.startPage(pageNum, pageSize);

        // 查询用户
        List<SysUser> sysUserList = sysUserMapper.findByPage(sysUserDto);
        startedPage.close();

        return new PageInfo<>(sysUserList);
    }

    /**
     * 添加用户
     *
     * @param sysUser 系统用户实体类
     */
    @Override
    public void saveSysUser(SysUser sysUser) {
        // 判断用户名不能为空
        String username = sysUser.getUserName();
        stringEmptyUtil.isEmpty(username, "用户名不能为空");

        SysUser dbUser = sysUserMapper.selectByUsername(sysUser.getUserName());
        // 如果用户已存在
        if (dbUser != null) {
            throw new BunnyException(ResultCodeEnum.USERNAME_IS_EXISTS);
        }
        // 设置密码加密
        String password = sysUser.getPassword();
        String encryptedPassword = MD5.encrypt(password);
        sysUser.setPassword(encryptedPassword);
        sysUser.setStatus(0);

        // 插入数据
        sysUserMapper.save(sysUser);
    }

    /**
     * 修改用户
     *
     * @param sysUser 系统用户实体类
     */
    @Override
    public void updateSysUser(SysUser sysUser) {
        stringEmptyUtil.isEmpty(sysUser.getId(), MessageConstant.UPDATE_ID_IS_NOT_EMPTY);
        sysUserMapper.updateSysUser(sysUser);
    }

    /**
     * 根据id删除用户
     *
     * @param userId 用户id
     */
    @Override
    public void deleteById(Long userId) {
        stringEmptyUtil.isEmpty(userId, MessageConstant.DELETE_ID_IS_NOT_EMPTY);
        sysUserMapper.deleteById(userId);
    }
}
